class OriginalPicture < ActiveRecord::Base
  belongs_to :artist
  belongs_to :license
  has_one :resource_picture
  
  before_destroy :destroy_with_file
  
  def validate
    errors.add(:filesize, 'size over(1MB)') if self.filesize > 1000000
  end
  
  def destroy_with_file
    PictureIO.original_picture_io.delete self.filename
    self.resource_picture.destroy
  end
  
  def dext
    self.ext.downcase
  end
  
  def filename
    "#{self.id}.#{self.dext}"
  end
  
  def mime_type
    "image/#{self.dext}"
  end
  
  def url
    '/original_pictures/' + filename
  end
  
  def store(rimg)
    bindata = rimg.to_blob
    PictureIO.original_picture_io.put bindata, self.filename
    res = if self.resource_picture
      self.resource_picture.store rimg
    else
      ResourcePicture.store(rimg, self)
    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.original_picture_io.get self.filename, subdir
  end
  
  def own? author
    return false unless author
    return false unless author.artist?
    self.artist_id == author.artist.id
  end
  
end
